@@ -9,7 +9,7 @@ import typer
 
 from . import __version__, builder, formatter, generator, logger, parser, updater
 from .exceptions import Ops2debError
-from .fetcher import DEFAULT_CACHE_DIRECTORY, Fetcher
+from .fetcher import DEFAULT_CACHE_DIRECTORY, set_cache_directory
 
 
 class DefaultCommandGroup(click.Group):
@@ -128,7 +128,7 @@ def default(
     debian_repository: str = option_debian_repository,
     workers_count: int = option_workers_count,
 ) -> None:
-    Fetcher.set_cache_directory(cache_directory)
+    set_cache_directory(cache_directory)
     try:
         blueprints = parser.parse(configuration_path)
         packages = generator.generate(blueprints, output_directory, debian_repository)
@@ -146,7 +146,7 @@ def generate(
     cache_directory: Path = option_cache_directory,
     debian_repository: str = option_debian_repository,
 ) -> None:
-    Fetcher.set_cache_directory(cache_directory)
+    set_cache_directory(cache_directory)
     try:
         blueprints = parser.parse(configuration_path)
         generator.generate(blueprints, output_directory, debian_repository)
@@ -187,10 +187,17 @@ def update(
         envvar="OPS2DEB_OUTPUT_FILE",
         help="Path to file where to save a summary of updated files.",
     ),
+    skip: Optional[List[str]] = typer.Option(
+        None,
+        "-s",
+        "--skip",
+        envvar="OPS2DEB_SKIP_BLUEPRINTS",
+        help="Name of blueprint that should not be updated. Can be used multiple times.",
+    ),
 ) -> None:
-    Fetcher.set_cache_directory(cache_directory)
+    set_cache_directory(cache_directory)
     try:
-        updater.update(configuration_path, dry_run, output_path)
+        updater.update(configuration_path, dry_run, output_path, skip)
     except Ops2debError as e:
         error(e, exit_code)
 
